/* rgbascale 1/19/96 Monty */

#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <math.h>
#include <malloc.h>
#include "./fft.c"

/* Range limiting */
inline int Range(int l, int x, int u){
  if(x>u)return(u);
  if(x>=l)return(x);
  return(l);
}

int main(int argc,char *argv[]){
  FILE *outfile=stdout;
  FILE *infile=stdin;
  int i,x,y;
  int xscale=-1, yscale=-1;
  /* method for scaling */
  int boxp=0,fourierp=0;

  for(i=1;i<argc;i++){
    if(isdigit(argv[i][0])){
      if(xscale==-1){
	char test[8];
	fscanf(infile,"%7c",test);
	test[7]='\0';
	if(strcmp(test,"RGBA32 ")){
	  fprintf(stderr,"rgbascale: input is not an RGBA32 file.\n");
	  exit(-1);
	}
      
	fscanf(infile,"%d %d%*1c",&x,&y);
      }
      
      if(argv[i][strlen(argv[i])-1]=='%'){
	if(xscale==-1)xscale=(float)x*(atof(argv[i]))/100.;
	yscale=(float)y*(atof(argv[i]))/100.;
      }else{
	if(xscale==-1)xscale=atoi(argv[i]);
	yscale=atoi(argv[i]);
      }
    }else{
      fclose(infile);
      infile=fopen(argv[i],"rb");
      if(infile==NULL){
	fprintf(stderr,"rgbascale: Unable to open input file.\n");
	exit(-1);
      }
    }
  }

  if(yscale==-1){
    fprintf(stderr,"rgbascale: No scaling argument[s] given.\n");
    exit(-1);
  }

  if(strcmp(argv[0]+strlen(argv[0])-1,"B")==0)boxp=1;
  if(strcmp(argv[0]+strlen(argv[0])-1,"F")==0)fourierp=1;

  if(x<xscale || y<yscale){
    fourierp=-1;
    boxp=0;
  }

  if(!boxp && !fourierp)
    boxp=-1;

  {
    int xi,yi,maxx,maxy;
    float *r,*g,*b,*a;
    float *rP,*gP,*bP,*aP;
    int ptr;

    maxx=x;
    if(x<xscale)maxx=xscale;
    maxy=y;
    if(y<yscale)maxy=yscale;

    r=rP=calloc(maxx*maxy,sizeof(float));
    g=gP=calloc(maxx*maxy,sizeof(float));
    b=bP=calloc(maxx*maxy,sizeof(float));
    a=aP=calloc(maxx*maxy,sizeof(float));

    for(yi=0;yi<y;yi++){
      ptr=maxx*yi;
      for(xi=0;xi<x;xi++){
	r[ptr]=(unsigned char)fgetc(infile);
	g[ptr]=(unsigned char)fgetc(infile);
	b[ptr]=(unsigned char)fgetc(infile);
	a[ptr]=(unsigned char)fgetc(infile);
	ptr++;
      }
    }

    if(fourierp){
      /* Unlike the box style thingies, we do x and y together for fourier */
      float *xlookup=calloc(2*maxx,sizeof(float));
      int *xifac=calloc(30,sizeof(int));
      float *ylookupf=calloc(2*maxy,sizeof(float));
      int *yifacf=calloc(30,sizeof(int));
      float *ylookupb=calloc(2*maxy,sizeof(float));
      int *yifacb=calloc(30,sizeof(int));
      long i,j;

      __ogg_fdrffti(x, xlookup, xifac);
      __ogg_fdrffti(y, ylookupf, yifacf);
      __ogg_fdrffti(yscale, ylookupb, yifacb);

      /* x is easy; y is gonna suck hard */
      for(i=0;i<y;i++){
	__ogg_fdrfftf(x,rP+i*maxx,xlookup,xifac);
	__ogg_fdrfftf(x,gP+i*maxx,xlookup,xifac);
	__ogg_fdrfftf(x,bP+i*maxx,xlookup,xifac);
	__ogg_fdrfftf(x,aP+i*maxx,xlookup,xifac);
      }

      __ogg_fdrffti(xscale, xlookup, xifac);
      for(i=0;i<y;i++){
	rP[x-2+i*maxx]=0;
	gP[x-2+i*maxx]=0;
	bP[x-2+i*maxx]=0;
	aP[x-2+i*maxx]=0;
	__ogg_fdrfftb(xscale,rP+i*maxx,xlookup,xifac);
	__ogg_fdrfftb(xscale,gP+i*maxx,xlookup,xifac);
	__ogg_fdrfftb(xscale,bP+i*maxx,xlookup,xifac);
	__ogg_fdrfftb(xscale,aP+i*maxx,xlookup,xifac);
	
	r=rP+i*maxx;
	g=gP+i*maxx;
	b=bP+i*maxx;
	a=aP+i*maxx;
	for(j=0;j<xscale;j++){
	  r[j]/=x;
	  g[j]/=x;
	  b[j]/=x;
	  a[j]/=x;
	}
      }

      /* Have to do a transpose */
      if(yscale!=y){
	float *temp=malloc(maxy*sizeof(float));
	for(i=0;i<xscale;i++){
	  r=rP+i;
	  g=gP+i;
	  b=bP+i;
	  a=aP+i;

	  /* red */
	  for(j=0;j<maxy;j++)
	    temp[j]=*(r+j*maxx);
	  __ogg_fdrfftf(y,temp,ylookupf,yifacf);
	  __ogg_fdrfftb(yscale,temp,ylookupb,yifacb);
	  for(j=0;j<yscale;j++)
	    *(r+j*maxx)=temp[j];

	  /* green */
	  for(j=0;j<maxy;j++)
	    temp[j]=*(g+j*maxx);
	  __ogg_fdrfftf(y,temp,ylookupf,yifacf);
	  __ogg_fdrfftb(yscale,temp,ylookupb,yifacb);
	  for(j=0;j<yscale;j++)
	    *(g+j*maxx)=temp[j];

	  /* blue */
	  for(j=0;j<maxy;j++)
	    temp[j]=*(b+j*maxx);
	  __ogg_fdrfftf(y,temp,ylookupf,yifacf);
	  __ogg_fdrfftb(yscale,temp,ylookupb,yifacb);
	  for(j=0;j<yscale;j++)
	    *(b+j*maxx)=temp[j];

	  /* alpha */
	  for(j=0;j<maxy;j++)
	    temp[j]=*(a+j*maxx);
	  __ogg_fdrfftf(y,temp,ylookupf,yifacf);
	  __ogg_fdrfftb(yscale,temp,ylookupb,yifacb);
	  for(j=0;j<yscale;j++)
	    *(a+j*maxx)=temp[j];

	  for(j=0;j<yscale;j++){
	    *(r+j*maxx)/=y;
	    *(g+j*maxx)/=y;
	    *(b+j*maxx)/=y;
	    *(a+j*maxx)/=y;
	  }
	}
	free(temp);
      }

    }else{
      int i;
      float racc,gacc,bacc,aacc;
      int uppery=0,lowery,ptr;
      float upperyf=1,loweryf,div=(float)yscale/y;

      for(yi=0;yi<y;yi++){
	float xleftf=1,xrightf;
	float div=(float)xscale/x,mul=(float)x/xscale;
	int xleft=0,xright,i;
	
	r=rP+yi*x;
	g=gP+yi*x;
	b=bP+yi*x;
	a=aP+yi*x;
	
	for(xi=0;xi<xscale;xi++){
	  
	  xrightf=(float)(xi+1)*mul;
	  xleftf=xrightf-mul;
	  xright=xrightf;
	  xrightf-=xright;
	  if(xright==x){
	    xrightf=1;
	    xright--;
	  }
	  
	  xleft=xleftf;
	  xleftf-=xleft;
	  xleftf=1-xleftf;

	  racc=xleftf*r[xleft]*a[xleft]+xrightf*r[xright]*a[xright];
	  gacc=xleftf*g[xleft]*a[xleft]+xrightf*g[xright]*a[xright];
	  bacc=xleftf*b[xleft]*a[xleft]+xrightf*b[xright]*a[xright];
	  aacc=xleftf*a[xleft]+xrightf*a[xright];
	  for(i=xleft+1;i<xright;i++){
	    racc+=r[i]*a[i];
	    gacc+=g[i]*a[i];
	    bacc+=b[i]*a[i];
	    aacc+=a[i];
	  }
	  if(aacc!=0){
	    r[xi]=rint(racc/aacc);
	    g[xi]=rint(gacc/aacc);
	    b[xi]=rint(bacc/aacc);
	  }else{
	    r[xi]=0;
	    g[xi]=0;
	    b[xi]=0;
	  }
	  a[xi]=rint(aacc*div);
	  
	}
      }

      for(yi=0;yi<yscale;yi++){
	upperyf=yi*(float)y/yscale;
	uppery=upperyf;
	upperyf-=uppery;
	upperyf=1-upperyf;
	
	loweryf=(yi+1)*(float)y/yscale;
	lowery=loweryf;
	loweryf-=lowery;
	if(lowery==y){
	  lowery--;
	  loweryf=1;
	}
	
	r=rP+uppery*maxx;
	g=gP+uppery*maxx;
	b=bP+uppery*maxx;
	a=aP+uppery*maxx;
	for(xi=0;xi<xscale;xi++){
	  ptr=0;
	  
	  racc=r[xi]*upperyf*a[xi];
	  gacc=g[xi]*upperyf*a[xi];
	  bacc=b[xi]*upperyf*a[xi];
	  aacc=a[xi]*upperyf;
	  for(i=uppery+1;i<lowery;i++){
	    ptr+=maxx;
	    racc+=r[xi+ptr]*a[xi+ptr];
	    gacc+=g[xi+ptr]*a[xi+ptr];
	    bacc+=b[xi+ptr]*a[xi+ptr];
	    aacc+=a[xi+ptr];
	  }
	  ptr+=maxx;
	  racc+=r[xi+ptr]*loweryf*a[xi+ptr];
	  gacc+=g[xi+ptr]*loweryf*a[xi+ptr];
	  bacc+=b[xi+ptr]*loweryf*a[xi+ptr];
	  aacc+=a[xi+ptr]*loweryf;

	  if(aacc!=0){
	    rP[xi+maxx*yi]=racc/aacc;
	    gP[xi+maxx*yi]=gacc/aacc;
	    bP[xi+maxx*yi]=bacc/aacc;
	    aP[xi+maxx*yi]=aacc*div;
	  }else{
	    rP[xi+maxx*yi]=0;
	    gP[xi+maxx*yi]=0;
	    bP[xi+maxx*yi]=0;
	    aP[xi+maxx*yi]=0;
	  }
	}
      }
    }

    fprintf(outfile,"RGBA32 %d %d\n",xscale,yscale);

    for(yi=0;yi<yscale;yi++){
      r=rP+yi*maxx;
      g=gP+yi*maxx;
      b=bP+yi*maxx;
      a=aP+yi*maxx;
      for(xi=0;xi<xscale;xi++)
	fprintf(outfile,"%c%c%c%c",(unsigned char)Range(0,rint(r[xi]),255),
		(unsigned char)Range(0,rint(g[xi]),255),
		(unsigned char)Range(0,rint(b[xi]),255),
		(unsigned char)Range(0,rint(a[xi]),255));
    }
  }
  fclose(infile);
  fclose(outfile);
  return(0);
}    





